<?php

namespace Litecms\Forever\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Forever\Http\Requests\ForveverRequest;
use Litecms\Forever\Interfaces\ForveverRepositoryInterface;
use Litecms\Forever\Models\Forvever;

/**
 * Resource controller class for forvever.
 */
class ForveverResourceController extends BaseController
{

    /**
     * Initialize forvever resource controller.
     *
     * @param type ForveverRepositoryInterface $forvever
     *
     * @return null
     */
    public function __construct(ForveverRepositoryInterface $forvever)
    {
        parent::__construct();
        $this->repository = $forvever;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Forever\Repositories\Criteria\ForveverResourceCriteria::class);
    }

    /**
     * Display a list of forvever.
     *
     * @return Response
     */
    public function index(ForveverRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Forever\Repositories\Presenter\ForveverPresenter::class)
                ->$function();
        }

        $forvevers = $this->repository->paginate();

        return $this->response->title(trans('forever::forvever.names'))
            ->view('forever::forvever.index', true)
            ->data(compact('forvevers'))
            ->output();
    }

    /**
     * Display forvever.
     *
     * @param Request $request
     * @param Model   $forvever
     *
     * @return Response
     */
    public function show(ForveverRequest $request, Forvever $forvever)
    {

        if ($forvever->exists) {
            $view = 'forever::forvever.show';
        } else {
            $view = 'forever::forvever.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('forever::forvever.name'))
            ->data(compact('forvever'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new forvever.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ForveverRequest $request)
    {

        $forvever = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('forever::forvever.name')) 
            ->view('forever::forvever.create', true) 
            ->data(compact('forvever'))
            ->output();
    }

    /**
     * Create new forvever.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ForveverRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $forvever                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('forever::forvever.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('forever/forvever/' . $forvever->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/forever/forvever'))
                ->redirect();
        }

    }

    /**
     * Show forvever for editing.
     *
     * @param Request $request
     * @param Model   $forvever
     *
     * @return Response
     */
    public function edit(ForveverRequest $request, Forvever $forvever)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('forever::forvever.name'))
            ->view('forever::forvever.edit', true)
            ->data(compact('forvever'))
            ->output();
    }

    /**
     * Update the forvever.
     *
     * @param Request $request
     * @param Model   $forvever
     *
     * @return Response
     */
    public function update(ForveverRequest $request, Forvever $forvever)
    {
        try {
            $attributes = $request->all();

            $forvever->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('forever::forvever.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('forever/forvever/' . $forvever->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('forever/forvever/' . $forvever->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the forvever.
     *
     * @param Model   $forvever
     *
     * @return Response
     */
    public function destroy(ForveverRequest $request, Forvever $forvever)
    {
        try {

            $forvever->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('forever::forvever.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('forever/forvever/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('forever/forvever/' . $forvever->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple forvever.
     *
     * @param Model   $forvever
     *
     * @return Response
     */
    public function delete(ForveverRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('forever::forvever.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('forever/forvever'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/forever/forvever'))
                ->redirect();
        }

    }

    /**
     * Restore deleted forvevers.
     *
     * @param Model   $forvever
     *
     * @return Response
     */
    public function restore(ForveverRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('forever::forvever.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/forever/forvever'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/forever/forvever/'))
                ->redirect();
        }

    }

}
